// Spherical quadrature filtering - Riesz triple                      //
// Isotropic filtering of a 2D color image and Riesz transform        //
// See [Felsberg & Sommer, The Monogenic Signal 
//      IEEE Trans. Signal Process., 2001, 49, 3136-3144]

// Code written by R. Soulard

function [s,sx,sy,H,Hx,Hy]=SQF(im,nu_min,nu_max)
  [M,N,K]=size(im);
  
  // Fourier coordinates:
  w1 =           [0:floor(M/2),-ceil(M/2)+1:-1]'*ones(1,N) *2*%pi/M;
  w2 = ones(M,1)*[0:floor(N/2),-ceil(N/2)+1:-1]            *2*%pi/N;
  nrm = sqrt(w1.^2+w2.^2); // |w|   in [  0  ;  pi*sqrt(2) ~ 4.44  ]
  
  // Piecewise construction of isotropic freq. response "H":
  if(nu_max>=3*nu_min) then
    a1 =      0      ;
    a2 =   2*nu_min  ;
  else
    d = (nu_max-nu_min)/2
    a1 =  nu_min-d   ;
    a2 =  nu_min+d  ;
  end
  H = ones(nrm);
  H( nrm <  a1 ) = 0; // zero LF coeffs
  if(a2>0) then
    idx = (  nrm>=a1 & nrm<=a2  );
    H(idx) = 0.5*(1-cos(%pi*  (nrm(idx)-a1)/(a2-a1)  ));
  end
  idx = (  nrm>=a2  );
  H(idx) = exp(-(   (nrm(idx)-a2)*log(2)/(2*(nu_max-a2))   ).^2);
  
  // Complex valued Riesz transform:
  Hriesz = H.*(w2-%i*w1)./(nrm+double(nrm==0));
  
  s = zeros(M,N,K);  sx = zeros(M,N,K);  sy = zeros(M,N,K);

  for i=1:K,
    sp = fft(im(:,:,i));
    s(:,:,i) = real(ifft( sp.*H ));
    sriesz = ifft( sp.*Hriesz );
    sx(:,:,i) = real(sriesz);
    sy(:,:,i) = imag(sriesz);
  end

  // Make freq. response's graphics for output:
  H = fftshift(H);
  Hx = fftshift( abs(imag(Hriesz)) );  
  Hy = fftshift( abs(real(Hriesz)) );  

endfunction
