// Code written by R. Soulard
// ------------------------------------------------------------------ //
// Process local frequency of a color image,                          //
// given phase and orientation values from monogenic analysis.        //
//                                                                    //
// uses function 'anglewrap()' from 'util.sci'                        //
// ------------------------------------------------------------------ //
function [nu,code] = inst_freq_2D(phaz,ori)
  // 'phaz': 2D matrix containing phase values
  // 'ori': 2D matrix containing orientation values
  // 'nu': 2D matrix containing measured local frequency
  // 'code': 2D matrix indicating 
  //             the considered local direction for frequency extraction
  // ---------------------------------------------------------------- //
  // QUANTIZING LOCAL DIRECTION:
  // NOTES:
  // Considering 'ori' is written in [0;2pi]:
  // The phase is supposed to evolve:
  // - vertically (along x-axis)             when ori=   0   OR  pi   |  code 1
  // - diagonally (top left to bottom right) when ori=  pi/4 OR 5pi/4 \  code 2
  // - horizontally (along y-axis)           when ori=  pi/2 OR 3pi/2 -  code 3
  // - diagonally (bottom left to top right) when ori= 3pi/4 OR 7pi/4 /  code 4
  //  
  // Convert the local main orientation 'ori' to 
  // 8-connexity Freeman-like code:
  // 6 5 4
  // 7   3
  // 8 1 2
  // Here codes 5 to 8 are merged with the opposite 1 to 4, so we use:
  // 2 1 4
  // 3   3
  // 4 1 2
  code = ceil(   anglewrap(ori+%pi/8,'[0;pi]') / (%pi/4)   );
  code(code==0) = 1;
  // ------------------------------------------------------------------------ //

  phaz = anglewrap(phaz,'[0;2pi]'); // (may be useless)

  // ------------------------------------------------------------------------ //
  // PROCESSING NU WITH QUANTIZED DIRECTION:
  nu = phaz*0;
  for k1=2:size(nu,1)-1,    // for each
    for k2=2:size(nu,2)-1,  // 'nu'-value
      // ------------------------------------------------- //
      // PICKING UP 3 CONSECUTIVE PHASE SAMPLES [a1 a2 a3] //
      // ------------------------------------------------- //
      a2 = phaz(k1,k2); // Middle sample
      // Select two opposite neighbours 'a1' and 'a3'
      // depending on quantized direction:
      select(code(k1,k2))
      case 1 then  // vertic | (diff w. above)
        a1 = phaz(k1-1,k2  );
        a3 = phaz(k1+1,k2  );
      case 2 then // diago  \ (diff w. top left)
        a1 = phaz(k1-1,k2-1);
        a3 = phaz(k1+1,k2+1);
      case 3 then // horiz  - (diff w. right
        a1 = phaz(k1  ,k2-1);
        a3 = phaz(k1  ,k2+1);
      case 4 then // diago  / (diff w. top right)
        a1 = phaz(k1-1,k2+1);
        a3 = phaz(k1+1,k2-1);
      end
      
      // We choose to exclude frequencies superior to "pi" rad/px,
      // even if such freq. are theoretically possible
      // in the HF corners of the 2D FFT.
      
      // Calculate first interval:
      a12 = exp(%i*a2)./exp(%i*a1);
      a12_rad = atan(imag(a12),real(a12)); // 1st interval in [-pi;pi]
      // Calculate 2nd interval:
      a23 = exp(%i*a3)./exp(%i*a2);
      a23_rad = atan(imag(a23),real(a23)); // 2nd interval in [-pi;pi]
      // Average phase difference:
      nu(k1,k2) = (abs(a12_rad) + abs(a23_rad))/2;

    end
  end

  // ---------------------------------------------------------------- //
  // APPLYING CORRECTIONS:
  // Correct diagonal phase difference measure:
  nu(code==2 | code==4) = nu(code==2 | code==4)/sqrt(2);

  // Correct the bias from orientation quantization 
  // for a linear phase model of the form:
  // phaz = nu * [ cos(ori)*x + sin(ori)*y ].
  // Differentiating along the QUANTIZED orientation "qori"
  // implies that the processed frequency is in fact "nu*cos(qori-rori)"
  // ===
  qori = (code-1)*%pi/4 ; // direction of differentiation (quantized)
  rori = anglewrap(ori+%pi/8,'[0;pi]') - %pi/8;// actual direction of signal
  bias = abs(cos(qori-rori)); // frequency factor due to their difference
  nu = nu ./ bias;

endfunction
