// 1D color oscillation.
// This script generates illustrations of 
// figure 1 from the ICIP paper

clear 
exec('util.sci'); // for 'modarg()' function
exec('lilly_ellipse.sci');

function [xx2,yy2,zz2]=rotate_curve(xx,yy,zz,ang,axi)
  // Rotate each point [xx(i),yy(i),zz(i)]
  // by angle 'ang' around axis 'axi'

  // Define rotation matrix:
  select(axi)
  case 'x' then rot_mat=[ 1 ,   0    ,   0     ;...
                          0 ,cos(ang),-sin(ang);...
                          0 ,sin(ang), cos(ang)  ];
  case 'y' then rot_mat=[ cos(ang), 0 ,sin(ang);...
                            0     , 1 ,  0     ;...
                         -sin(ang), 0 ,cos(ang)  ];
  case 'z' then rot_mat=[ cos(ang),-sin(ang),  0 ;...
                          sin(ang), cos(ang),  0 ;...
                             0    ,   0     ,  1   ];
  else disp('rotate_curve : no such axis!');
  end    
  // Apply rotation on every sample:
  xx2=xx; yy2=yy; zz2=zz;
  for i=1:length(xx),
    tmp = [xx(i);yy(i);zz(i)];
    tmp = rot_mat * tmp;
    xx2(i) = tmp(1);
    yy2(i) = tmp(2);
    zz2(i) = tmp(3);
  end
endfunction



// ******************************************************** //
// ********** DATA **************************************** //
// ******************************************************** //

t = 0:0.2:2*%pi;// time scale in seconds
t = [t,2*t($)-t($-1)]; // add a last point to close the drawn line

// Define color 1D Fourier atom coefficients:
As = [ 0.70 , 0.18 , 0.36 ];     // marginal amplitudes
Ps = [-0.63 ,-0.10 ,-0.03 ]*%pi; // marginal phases
w = 1; // global frequency in radians per seconds

// Process color oscillation:
r = As(1)*cos(w*t+Ps(1));
g = As(2)*cos(w*t+Ps(2));
b = As(3)*cos(w*t+Ps(3));

// Process ellipse parameters:
[kapp,lambd,phi,alph,bet,thet] = lilly_ellipse( As(1)*cos(Ps(1)) , ...
                                                As(2)*cos(Ps(2)) , ...
                                                As(3)*cos(Ps(3)) , ...
                                                As(1)*sin(Ps(1)) , ...
                                                As(2)*sin(Ps(2)) , ...
                                                As(3)*sin(Ps(3))      );
// Note : the actual time-varying phase data is "phi+w*t"
// every other parameter is constant

// ******************************************************** //
// ********** GRAPHICS ************************************ //
// ******************************************************** //

// ------------------------------------------------------ //
// Imshow one line:
// ------------------------------------------------------ //
tmp = ones(1,1);
img1 = 0.5+0.5*cimg( tmp*r , tmp*g , tmp*b );
imshow(img1);
// ------------------------------------------------------ //
// Imshow example plane wave:
// ------------------------------------------------------ //
nper = 5;
tt = 0:0.1:nper*2*%pi;// time scale in seconds
xxx = tt'*ones(tt);
yyy = ones(tt')*tt;
ori = %pi/3;
ttt = cos(ori).*xxx + sin(ori).*yyy;
// Process plane wave:
rrr = As(1)*cos(w*ttt+Ps(1));
ggg = As(2)*cos(w*ttt+Ps(2));
bbb = As(3)*cos(w*ttt+Ps(3));
img2 = 0.5+0.5*cimg(rrr,ggg,bbb);
imshow(img2);
// ------------------------------------------------------ //
// 1D SIGNAL :
// ------------------------------------------------------ //
scf(1);clf(1);fff=gcf();
fff.axes_size=[280,70];
fff.figure_position = [0,0];

red_idx = color(255,0,0);
gre_idx = color(0,180,0);
blu_idx = color(0,0,255);
mx = 1*max(abs([r;g;b]));

plot2d([t;t;t]',[r;g;b]',style=[red_idx,gre_idx,blu_idx]);
a=gca();
a.auto_ticks=["off","off","off"];
//a.x_ticks = tlist(["ticks" "locations", "labels"],...
//                  [0;%pi;2*%pi], [" ";"pi";"2pi"]);
//a.y_ticks = tlist(["ticks" "locations", "labels"],...
//                  [-0.5;0;0.5], ["-.5";"0";".5"]);
a.x_ticks = tlist(["ticks" "locations", "labels"],...
                  [0;%pi;2*%pi], [" ";" ";" "]);
a.y_ticks = tlist(["ticks" "locations", "labels"],...
                  [-0.5;0;0.5], [" ";" ";" "]);
//a.sub_ticks = [3,4];
a.sub_ticks = [3,0];
a.x_location = "middle";
a.tight_limits = "on";
a.data_bounds = [t(1),-1.1*mx;t($),1.1*mx];
a.margins = [0.01,0,0,0];

a.children(1).children(1).thickness = 2;
a.children(1).children(2).thickness = 2;
a.children(1).children(3).thickness = 2;
// ------------------------------------------------------ //
// ------------------------------------------------------ //
// ------------------------------------------------------ //

// ------------------------------------------------------ //
// 3D ELLIPSE :
// ------------------------------------------------------ //
scf(2);clf(2);fff=gcf();
fff.axes_size=[210,130];
fff.figure_position = [300,0];
// ------------------------------------------------------ //
// Prepare:
// ------------------------------------------------------ //
mx = 1.1*max(abs([r;g;b]));
mxtop = 1.3*max(abs([b]));
// Get RGB colors of every point of the atom:
// And add them to colormap (figure must be active):
fg=[];
for i=1:length(t)-1,
  rr = round((0.5*r(i)+0.5)*255);// map [-1;1]^3 into the RGB cube
  gg = round((0.5*g(i)+0.5)*255);// map [-1;1]^3 into the RGB cube
  bb = round((0.5*b(i)+0.5)*255);// map [-1;1]^3 into the RGB cube
  fg = [fg, color(rr,gg,bb) ];
end
grey1 = color(128,128,128);
grey2 = color(220,220,220);
// ------------------------------------------------------ //
// * Plot:
// ------------------------------------------------------ //
//// Plot phase:
//// (show where t=0)
//param3d1([0;r(1)],[0;g(1)] , list([0;b(1)] , [grey1]) );
//// Plot normal vector:
//n=nvec*0.2/norm_nvec; // (resize nvec)
//param3d1([0;n(1)],[0;n(2)] , list([0;n(3)] , [grey1]) );
// * Plot background marginal ellipses:
mxx = ones(r')*(mx); // ("blank" vector)
mxx_top = ones(r')*(mxtop); 
param3d1([mxx,r',r'],[g',-mxx,g'],...
         list([b',b',-mxx_top] , [grey2,grey2,grey2]) );
         
mx1 = kapp*sqrt(1+lambd);// semi-major axis (noted 'a')
mx2 = kapp*sqrt(1-lambd);// semi-minor axis (noted 'b')
// * Curve 1 : Major axis
xx1 = [-mx1;mx1]; 
yy1 = [  0 ; 0 ]; 
zz1 = [  0 ; 0 ]; 
[xx1,yy1,zz1]=rotate_curve(xx1,yy1,zz1,thet,'z');// Rotate to the
[xx1,yy1,zz1]=rotate_curve(xx1,yy1,zz1,bet ,'x');// plane where
[xx1,yy1,zz1]=rotate_curve(xx1,yy1,zz1,alph,'z');// the ellipse is.
// * Curve 2 : Minor axis
xx2 = [  0 ; 0 ]; 
yy2 = [-mx2;mx2];
zz2 = [  0 ; 0 ];
[xx2,yy2,zz2]=rotate_curve(xx2,yy2,zz2,thet,'z');// Rotate to the
[xx2,yy2,zz2]=rotate_curve(xx2,yy2,zz2,bet ,'x');// plane where
[xx2,yy2,zz2]=rotate_curve(xx2,yy2,zz2,alph,'z');// the ellipse is.
param3d1(  [xx1,xx2] , [yy1,yy2] , list([zz1,zz2],[1,1])  );

// * Curve 3 : Bounding rectangle
xx3 = [-mx1; mx1;mx1;-mx1;-mx1]; // Temporary
yy3 = [-mx2;-mx2;mx2; mx2;-mx2]; // horizontal
zz3 = [  0 ;  0 ; 0 ;  0 ;  0 ]; // bounding plane
[xx3,yy3,zz3]=rotate_curve(xx3,yy3,zz3,thet,'z');// Rotate to the
[xx3,yy3,zz3]=rotate_curve(xx3,yy3,zz3,bet ,'x');// plane where
[xx3,yy3,zz3]=rotate_curve(xx3,yy3,zz3,alph,'z');// the ellipse is.
param3d1( xx3,yy3,list(zz3,1) );

// * Plot coloured main ellipse: 
param3d1([r(1:$-1);r(2:$)],[g(1:$-1);g(2:$)],...
         list([b(1:$-1);b(2:$)],fg) , flag=[3,0] );
// (this is a set of two-point polylines 
//     with independent colour)
// ------------------------------------------------------ //
// * Change graphical attributes:
// ------------------------------------------------------ //
a=gca();

// * Main ellipse:
for i=1:length(t)-1, // (change thickness)
  a.children(1).children(i).thickness = 5;
end

// * Local plane:
a.children(2).line_style = 3; // (dash)

// * Major/Minor axes:
for i=1:2, 
  a.children(3).children(i).line_style = 3; //(dash)
end

// * Background ellipses:
for i=1:3, // (make them filled)
  a.children(4).children(i).polyline_style = 5;
end
//// * Normal vector:
//a.children(4).polyline_style = 4; // (arrow)

a.x_label.visible = "off";
a.y_label.visible = "off";
a.z_label.visible = "off";
//a.x_label.text = "R";  a.x_label.position = [-0.9,0.4];
//a.y_label.text = "G";  a.y_label.position = [0.9,-0.5];
//a.z_label.text = "B";  a.z_label.position = [0.1,2];
a.auto_ticks=["off","off","off"];
//a.x_ticks = tlist(["ticks" "locations", "labels"],...
//                  [-0.5;0;0.5], ["-.5";"R";".5"]);
//a.y_ticks = tlist(["ticks" "locations", "labels"],...
//                  [-0.5;0;0.5], ["-.5";"G";".5"]);
//a.z_ticks = tlist(["ticks" "locations", "labels"],...
//                  [-0.2;0;0.2], ["-.2";"B";".2"]);
//a.x_ticks = tlist(["ticks" "locations", "labels"],...
//                  [-0.5;0;0.5], [" ";" ";" "]);
//a.y_ticks = tlist(["ticks" "locations", "labels"],...
//                  [-0.5;0;0.5], [" ";" ";" "]);
//a.z_ticks = tlist(["ticks" "locations", "labels"],...
//                  [-0.2;0;0.2], [" ";" ";" "]);
a.box    = "on";
a.filled = "off";
a.sub_ticks = [4,4,1];
a.isoview      = "on";
a.cube_scaling = "off";
//a.rotation_angles = [80,419];
a.rotation_angles = [79.75,476.5];
a.tight_limits = "on";
a.data_bounds = [-mx,-mx,-mxtop;mx,mx,mxtop];
//a.margins = [0.001,0.1,0,0.1]; // gauche droite haut bas
a.margins = [0,0,0,0]; // gauche droite haut bas
a.hidden_axis_color = grey1;// make hidden axis grey
a.hiddencolor       = grey1;// make hidden grey (?)
a.background        = grey1;// grey background (invisible)
// ------------------------------------------------------ //
// ------------------------------------------------------ //
// ------------------------------------------------------ //


// ------------------------------------------------------ //
// 2D ELLIPSE :
// ------------------------------------------------------ //

scf(3);clf(3);fff=gcf();
fff.axes_size=[200,120];
fff.figure_position = [600,0];
// ------------------------------------------------------ //
// * Prepare colormap:
// ------------------------------------------------------ //
// Get RGB colors of every point of the atom:
// And add them to colormap (figure must be active):
fg=[];
for i=1:length(t)-1,
  rr = round((0.5*r(i)+0.5)*255);// map [-1;1]^3 into the RGB cube
  gg = round((0.5*g(i)+0.5)*255);// map [-1;1]^3 into the RGB cube
  bb = round((0.5*b(i)+0.5)*255);// map [-1;1]^3 into the RGB cube
  fg = [fg, color(rr,gg,bb) ];
end
grey1 = color(128,128,128);
grey2 = color(200,200,200);
// ------------------------------------------------------ //
// Compute data:
// ------------------------------------------------------ //
// Ellipse (a1Dx,a1Dy):
[ xx , yy ,zz]=rotate_curve( r, g, b,-alph,'z');// Rotate ellipse 
[a1Dx,a1Dy,zz]=rotate_curve(xx,yy,zz,-bet ,'x');// to the x-y plane
// Phase line (phix,phiy):
phix = [0;a1Dx(1)];
phiy = [0;a1Dy(1)];
// Major axes lines (ma1x,ma1y) and (ma2x,ma2y):
mx1 = kapp*sqrt(1+lambd);// semi-major axis (noted 'a')
mx2 = kapp*sqrt(1-lambd);// semi-minor axis (noted 'b')
[ma1x,ma1y,zz]=rotate_curve(  [0 0] , [-mx2 mx2] ,[0,0],thet,'z');// Rotate 
[ma2x,ma2y,zz]=rotate_curve(  [-mx1 mx1] , [0 0] ,[0,0],thet,'z');// Rotate 
// ------------------------------------------------------ //
// * Plot:
// ------------------------------------------------------ //
plot2d(phix,phiy,1,"065");  // Plot phase (show where t=0)
plot2d([ma1x;ma2x]',[ma1y;ma2y]',[1,1],"065"); // Plot major axes
// Plot coloured main ellipse: 
plot2d([a1Dx(1:$-1);a1Dx(2:$)],[a1Dy(1:$-1);a1Dy(2:$)], fg , "065");
// (this is a set of two-point polylines with independent colour)
// ------------------------------------------------------ //
// * Change graphical attributes:
// ------------------------------------------------------ //
a=gca();
// * Main ellipse:
for i=1:length(t)-1, // (change thickness)
  a.children(1).children(i).thickness = 5;
end
// * Major axes:
tmp = a.children(2); 
tmp.children(1).line_style = 3; // (dash)
tmp.children(2).line_style = 3; // (dash)
tickx = ceil(mx1*10)/10; // Place tick on the largest decimal
ticky = ceil(mx2*10)/10; // Place tick on the largest decimal
a.auto_ticks=["off","off","off"];
a.x_ticks = tlist(["ticks" "locations", "labels"],[-tickx;tickx],[" ";" "]);
a.y_ticks = tlist(["ticks" "locations", "labels"],[-ticky;ticky],[" ";" "]);
a.box    = "off";
a.filled = "off";
a.sub_ticks = 20*[tickx,ticky]-1;
a.isoview      = "on";
a.cube_scaling = "off";
a.tight_limits = "on";
a.data_bounds = 1.1*[-mx1,-mx2;mx1,mx2];
a.margins = [0,0,0,0];
// ------------------------------------------------------ //
// ------------------------------------------------------ //
// ------------------------------------------------------ //
disp("[As;Ps]:");disp([As;Ps]);
disp("thet , bet , alph , phi , lambd , kapp");
disp([thet,bet,alph,phi,lambd,kapp]);

function writeimages()
  imwrite(img1,'fig_ell_1D.png');
  imwrite(img2,'fig_ell_planewave.png');
  xs2pdf(1,'fig_ell_1D');
  xs2pdf(2,'fig_ell_3D');
  xs2pdf(3,'fig_ell_2D');
endfunction
//writeimages();

disp('done.');
