// Show Elliptical monogenic color analysis
// (figure 2 in ICIP paper)

clear
exec('util.sci');
exec('SQF.sci');
exec('gaussian.sci');
exec('tensor.sci');
exec('lilly_ellipse.sci');
exec('rsimg.sci');

im = double(imread('face8.png'))/255;
nu_min = 0.15; // min freq. for isotropic bandpass (rad./px)
nu_max = 0.60; // max freq. for isotropic bandpass (rad./px)

// *** Isotropic filtering and Riesz transform:                       //
//                                  (by Spherical Quadrature Filters) //

[s,sx,sy,H,Hx,Hy] = SQF(im,nu_min,nu_max);
// s: filtered image
// sx,sy: Riesz transform of each color channel of s
// H,Hx,Hy: frequency responses of the 3 filters


// *** Get local orientation:                                         //
//                            (from the Riesz-based structure tensor) //

gauss_sigma = 1; // width of smoothing kernel
[n,lambdaP,coher,ori] = tensor(sx,sy,gauss_sigma);
clear n lambda coher // (unused tensor features)


// *** Process directional color Riesz transform: 

sh = ( sx .* cimg(cos(ori)) + sy .* cimg(sin(ori)) );


// *** Get ellipse parameters:

[kap,lam,phi,a1,a2,a3] = lilly_ellipse( s , sh );
A = kap*sqrt(2); // amplitude


// *** Make illustrations:

m = max(abs([s,sx,sy,sh])); // normalization constant
A =sin(0.5*%pi*A/max(A)).^2; // Enhance contrast

illus = list();
names = list();
illus($+1) = im                   ;  names($+1) = 'im'   ;
illus($+1) = rsimg(s    ,'band',m);  names($+1) = 's'    ;
illus($+1) = rsimg(sx   ,'band',m);  names($+1) = 'sx'   ;
illus($+1) = rsimg(sy   ,'band',m);  names($+1) = 'sy'   ;
illus($+1) = rsimg(sh   ,'band',m);  names($+1) = 'sh'   ;
illus($+1) = rsimg(ori  ,'dir' ,A);  names($+1) = 'ori'  ;
illus($+1) = rsimg(A    ,'amp'   );  names($+1) = 'A'  ;
illus($+1) = rsimg(1-lam,'amp' ,1);  names($+1) = '1-lam'  ;
illus($+1) = rsimg(phi  ,'phaz',A);  names($+1) = 'phi' ;
illus($+1) = rsimg(a1   ,'dir' ,A);  names($+1) = 'a1';
illus($+1) = rsimg(a2   ,'dir' ,A);  names($+1) = 'a2' ;
illus($+1) = rsimg(a3   ,'dir' ,A);  names($+1) = 'a3';
illus($+1) = rsimg(H    ,'amp' ,1);  names($+1) = 'H'    ;
illus($+1) = rsimg(Hx   ,'amp' ,1);  names($+1) = 'Hx'   ;
illus($+1) = rsimg(Hy   ,'amp' ,1);  names($+1) = 'Hy'   ;

// *** imshow()

imshow([ illus(1),illus(2),illus(3),illus(4),illus(5);...
         illus(6),illus(7),illus(8),illus(9),illus(10);...
         illus(11),illus(12),illus(13),illus(14),illus(15) ]);

// ****************************************************************** //
// *** Write figures and legends:

for i=1:length(illus),
  imwrite(illus(i),'fig_cmonog_'+names(i)+'.png');
end
N = size(im,2); // width
K = ceil(N/40); // height
lgd = rsimg(  ones(K,1)*([N-1:-1:0]/(N-1))   ,'amp' ,1);
imwrite(lgd,'fig_cmonog_lgd_1-lam.png');
lgd = rsimg(  ones(K,1)*([ 0:N-1  ]/(N-1))   ,'amp' ,1);
imwrite(lgd,'fig_cmonog_lgd_A.png');
lgd = rsimg(  ones(K,1)*(  ([0:N-1]-floor(N/2))*%pi/N  )   ,'dir');
imwrite(lgd,'fig_cmonog_lgd_ori.png');
lgd = rsimg(  ones(K,1)*(  [0:N-1]*%pi/(2*N)  )   ,'phaz');
imwrite(lgd,'fig_cmonog_lgd_phi.png');



// ****************************************************************** //
disp('Done.');
// ****************************************************************** //
