// Code written by R. Soulard

// --------------------------------------------------- //
// -------   DEBUG   --------------------------------- //
// --------------------------------------------------- //
// subroutine to disp function error message :
function printerror(fnam,msg)
	// fnam (string) : name of the function
	// msg (string) : message
	printf("error with function %s: %s\n",fnam,msg);
endfunction
// --------------------------------------------------- //
// -----  FFT  --------------------------------------- //
// --------------------------------------------------- //
function y=rs_fft(x) 
  //y = mfft(x,-1,size(x)); // Scilab 5.4 and older
  y = fft(x); // for Scilab 5.5
endfunction
function y=rs_ifft(x)
  //y = mfft(x,+1,size(x)); // Scilab 5.4 and older
  y = ifft(x); // for Scilab 5.5
endfunction
function y=rs_ifftshift(x) // seems obsolete since Scilab 5.5 features identical ifftshift now
  // replicates MATLAB ifftshift for matrices,
  // handles the odd/even particular cases.
  y = x;
  [M,N]=size(y);
  an = floor(N/2);
  am = floor(M/2);
  y = [ y((am+1):M,(an+1):N) , y((am+1):M,1:an) ;...
        y( 1:am , (an+1):N) , y( 1:am ,1:an) ];
endfunction
// --------------------------------------------------- //
// -----   COMPLEX ALGEBRA   ------------------------- //
// --------------------------------------------------- //
function [m,p]=modarg(varargin)
  // compute modulus and argument of complex valued matrix 'c'
  // (set singular values to zero)
  if(length(varargin)==1) then
    c = varargin(1);
  elseif(length(varargin)==2) then
    c = varargin(1) + %i*varargin(2);
  else
    printerror("modarg","args must be a complex matrix or the separate real and imaginary parts of it");
  end
  m = abs(c);
  r = real(c);
  i = imag(c);
  p = atan(i,r);
  idx = ( isnan(c) | isinf(r) | isinf(i) | m==0 | isnan(p) );
  m(idx)=0;
  p(idx)=0;
endfunction
// --------------------------------------------------- //
function b = anglewrap(a,opt)
	// wraps an angle data 'a' into some restricted interval 
	// by adaptively adding or substracting %pi or 2*%pi
  // usage : b=anglewrap(a,'[-pi/2;pi/2]');
  pipi = a - 2*%pi*round(a/(2*%pi)); // set a in [-pi;pi]
  select opt,
  case '[-pi;pi]' then b = pipi;
  case '[0;2pi]' then b=pipi; b(b<0)=b(b<0)+2*%pi;
  case '[-pi/2;pi/2]' then b=pipi;
    b(b<-%pi/2)=b(b<-%pi/2)+%pi;
    b(b>+%pi/2)=b(b>+%pi/2)-%pi;
  case '[0;pi]' then b=pipi; b(b<0)=b(b<0)+%pi;
  else b=a; printf('angle_wrap : option not valid!');
  end
endfunction
// --------------------------------------------------- //
// -----   COLOR MATRICES   -------------------------- //
// --------------------------------------------------- //
// subroutine to test matrices:
function b = isscalarmat(m)
  b = ( length(size(m))==2 );
endfunction
function b = isvectormat(m)
  b = ( length(size(m))==3 );
endfunction
function b = is3Dvectormat(m)
  if(isvectormat(m)) then
    b = size(m,3)==3;
  else
    b = %F;
  end
endfunction
// --------------------------------------------------- //
function v=cimg(varargin)
  // construct a 3D vector valued matrix 'v'
  // If 'varargin' is one scalar matrix, then the three 
  // components of 'v' will all be equal to 'm'.
  // If 'varargin' is made of 3 scalar matrices 'r,g,b', 
  // then 'v' will be made of these.
  if(length(varargin)==1) then
    r = varargin(1);
    if(~isscalarmat(r)) then
      printerror("cimg","scalar matrix expected");
    end
    g = r;
    b = r;
  elseif(length(varargin)==3) then
    r = varargin(1);
    g = varargin(2);
    b = varargin(3);
    if(~isscalarmat(r)|~isscalarmat(g)|~isscalarmat(b)) then
      printerror("cimg","args must be scalar matrices");
    end
  else
    printerror("cimg","1 or 3 args expected");
  end
	// make a 3D vector valued MxNx3 matrix 'v' from 3 MxN matrices 'r', 'g', 'b'. 
  v = zeros(size(r,1),size(r,2),3);
  v(:,:,1)=r;
  v(:,:,2)=g;
  v(:,:,3)=b;
endfunction
// --------------------------------------------------- //
function [r,g,b]=icimg(v)
	// make a 3D vector valued MxNx3 matrix 'v' from 3 MxN matrices 'r', 'g', 'b'. 
  r=v(:,:,1);
  g=v(:,:,2);
  b=v(:,:,3);
endfunction
// --------------------------------------------------- //
function n=csum(v)
	// compute the sum of the components of the vector elements of matrix v
  if(isscalarmat(v)) then
    n=v;
  elseif(isvectormat(v)) then
    K = size(v,3);
    n = v(:,:,1);
    for i=2:K;
      n = n + v(:,:,i);
    end
  else
    printerror("csum","matrix should be scalar or vector valued");
  end
endfunction
// --------------------------------------------------- //
function n=cnorm(v)
	// compute Euclidean norm of vector valued matrix v
  n = sqrt(csum(abs(v).^2));
endfunction
// --------------------------------------------------- //
// routines for handling lists of matrices (e.g. pyramids) 
// being either scalar valued or 3D vector valued:
// --------------------------------------------------- //
// --- Conversion list/vector 
function [v,dims] = vect_pyramid(p)
  // Convert a list of matrices into a vector
  // plus a description of matrices dimensions
  N = length(p)
  v = [];
  dims = [];
  for i=1:N,
    x = p(i);
    v = [v;x(:)];
    dims = [dims;[size(x,1),size(x,2)]];
  end
endfunction
//
function p = i_vect_pyramid(v,dims)
  // Convert a vector with a description of 
  // matrices dimensions into a list of matrices
  p = list();
  N = size(dims,1);
  for i=1:N,
    MM = dims(i,1);
    NN = dims(i,2);
    v2 = v(1:(MM*NN));
    p($+1) = matrix(v2,MM,NN);
    v = v(MM*NN+1:$);
  end
endfunction
// --- Conversion scalar/3D vector
function p=cimgpy(r,g,b)
  // create a list of 3D vector valued matrices p
  // from 3 lists of scalar valued matrices r,g,b  
  p = list();
  for i=1:length(r);
    p($+1) = cimg(r(i),g(i),b(i));
  end
endfunction
//
function [r,g,b]=icimgpy(p)
  // extract 3 lists of scalar valued matrices r,g,b  
  // from a list of 3D vector valued matrices p
  r = list();
  g = list();
  b = list();
  for i=1:length(p);
    tmp = p(i);
    r($+1) = tmp(:,:,1);
    g($+1) = tmp(:,:,2);
    b($+1) = tmp(:,:,3);
  end
endfunction
// is the list scalar valued or 3D vector valued?
function col = is3Dvectorpy(p)
  // test if the matrices of the list 'p' 
  // are 3D vector valued.
  // return an error if scalar and vector are mixed
  for i=1:length(p);
    tmp=p(i);
    if(i==1) then
      col = is3Dvectormat(tmp); // is the matrix 3D valued?
    else
      if(col~=is3Dvectormat(tmp)) then
        printerror("isvectorpy","the list mixes scalar and vector valued matrices");
      end
    end
  end
endfunction
// --------------------------------------------------- //
// --- utils :
// --------------------------------------------------- //
function m = maxabspy(p)
  // Finds the max absolute value of the whole list of matrices
  // handle 3D vector valued case
  if(is3Dvectorpy(p)) then
    [r,g,b]=icimgpy(p);
    m = max([maxabspy(r),maxabspy(g),maxabspy(b)]);
  else
    [xV,dims] = vect_pyramid(p);
    m = max(abs(xV));
  end
endfunction
// --------------------------------------------------- //
function p2 = initclonepy(p,val)
  // create a new pyramid of same dimensions of 'p'
  // with initialization value 'val' everywhere
  p2 = p;
  for i=1:length(p);
    p2(i) = p(i)*0 + val;
  end
endfunction
// --------------------------------------------------- //
// Display a pyramid as a color bitmap :
// --------------------------------------------------- //
function y=layoutpy(L)
  // create a pyramid layout
  // L is a list of J+1 color images illustrating subbands
  // (filling color is white)
  // ----------------------------------------
  // Top block [y]:
  y = [];
  for i=length(L):-1:2,
    s = L(i); // color subband
    if(length(size(y))>2) then
      y = [s , y]; // add subband to the left
    else
      y = [s]; // add subband to the left
    end
    if(i~=2) then // add blank patch on top of subband
      blk = ones(y(:,:,1)); // blank patch
      blk=cimg(blk);
      y = [blk ; y];   // add blank patch to the top
    end
  end
  // ----------------------------------------
  // Add big subband at the bottom of [y] :
  HF = L(1);
  [ M, N,tmp]=size(HF);// size of big subband
  [yM,yN,tmp]=size(y);// size of [y]
  // Create blank patch on the right of [y] to reach same width than big subband
  blk = ones( yM , N-yN );
  blk=cimg(blk);
  // Form the final layout :
  y = [ [y,blk] ; HF ];
endfunction
// --------------------------------------------------- //

